﻿#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# QuickRecon v0.1.1
# Copyright (C) 2011 Filip Szymański <f.szymanski@hotmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# For help run 'python quickrecon.py'
#
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
# Please note that this tool was created and published for educational purposes only.
#
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import getopt
import httplib
import random
import re
import socket
import StringIO
import sys
import time
import urllib2

try:
	import dns.resolver, dns.query, dns.zone
except ImportError:
	print 'Please install \'dnspython\' from http://www.dnspython.org/.'
	sys.exit(1)

buffer = StringIO.StringIO()

def usage():
	print '\n ****************************'
	print ' * QuickRecon v0.1.1        *'
	print u' * Coded by Filip Szymański *'
	print ' * f.szymanski@hotmail.com  *'
	print ' ****************************\n'
	print ' Usage: quickrecon [options]\n'
	print '                      -m, --mode= {s, z, e}     Working mode.\n'
	print '                                      s         Find subdomains.'
	print '                                      z         Zone transfer.'
	print '                                      e         Extract emails from Google and Bing.\n'
	print '                      -d, --domain=             Domain name.'
	print '                      -i, --input=              Input file.'
	print '                      -o, --output=             Output file.\n'
	print ' Examples:            quickrecon.py -m s -d example.com -o out.log'
	print '                      quickrecon.py -m s -d example.com -i your_subdomains.txt'
	print '                      quickrecon.py -m z -d example.com'
	print '                      quickrecon.py -m e -d example.com -o emails.log\n'

def find_subdomains(ifile):
	test_wildcard()
	
	found = 0

	try:
		file = open(ifile, 'r')
		print ' / SUBDOMAIN: /\n'
		buffer.write(' / SUBDOMAIN: /\n\n')

		for line in file.readlines():
			sub = line.rstrip('\n')
			subdomain = sub + '.' + domain

			try:
				ip = socket.gethostbyname(subdomain)
				found = found + 1
			#	reverse = socket.gethostbyaddr(ip)
				print subdomain #, ip, reverse[0]
				buffer.write(subdomain + '\n')
			except:
				pass

		print '\n  Found %s subdomain(s).' % found
		buffer.write('\n  Found ' + str(found) + ' subdomain(s).\n')

	except IOError:
		print 'No such file or directory: \'%s\'.' % ifile
		sys.exit(1)
	finally:
		file.close()

def test_wildcard():
	container = []
	scode = []
	charset = 'abcdefghijklmnopqrstuvwxyz0123456789'

	for count in range(3):
		rcharset = random.sample(charset, random.randint(6, 12))
		rsub = ''.join(rcharset)
		rsubdomain = rsub + '.' + domain
		container.append(rsubdomain)

	for tsubdomain in container:
		try:
			conn = httplib.HTTPConnection(tsubdomain)
			conn.putrequest('GET', '/')
			conn.putheader('User-Agent', 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)')
			conn.endheaders()
			response = conn.getresponse()
		#	print response.msg
			scode.append(response.status)
		except:
			pass

	if len(scode) > 0:
		print 'The target domain has a DNS wildcard configuration.'
		sys.exit(1)
	else:
		pass

def zone_transfer():
	ns = []
	data = ''

	print ' / ZONE TRANSFER: /\n'
	buffer.write(' / ZONE TRANSFER: /\n\n')

	nsrecords = dns.resolver.query(domain, 'NS')
	for server in nsrecords:
		ns.append(str(server))
		
	for target in ns:
		try:
			zt = dns.zone.from_xfr(dns.query.xfr(target.rstrip('.'), domain))
			names = zt.nodes.keys()
			names.sort()
			for name in names:
				data += zt[name].to_text(name)
				print re.sub(' ', '   ', zt[name].to_text(name))
				buffer.write(re.sub(' ', '   ', zt[name].to_text(name)) + '\n')
		except:
			pass

	if data == '':
		print '  Zone transfer is not possible.'
		buffer.write('  Zone transfer is not possible.\n')
	else:
		pass

def generate_urls():
	global urls
	urls = []

	for ngoogle in range(0, 100, 10):
		urls.append('http://www.google.com/search?hl=en&ie=UTF-8&num=10&q=%40' + domain + '&start=' + str(ngoogle))
		urls.append('http://groups.google.com/groups?hl=en&ie=UTF-8&num=10&q=%40' + domain + '&start=' + str(ngoogle))

	for nbing in range(1, 101, 10):
		urls.append('http://www.bing.com/search?q=%40' + domain + '&go=&count=10&first=' + str(nbing))

def find_emails(urls):
	patterns = [re.compile('[\w.]+@' + string + domain) for string in ['<b>', '<strong>']]
	results = {}

	print '  / EMAIL: /\n'
	buffer.write(' / EMAIL: /\n\n')

	for url in urls:
		request = urllib2.Request(url)
		request.add_header('User-agent', 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)')
		response = urllib2.urlopen(request)
		data = response.read()
		for pattern in patterns:
			rawemails = re.findall(pattern, data)
			for email in rawemails:
				uemail = re.sub('<strong>|<b>', '', email)
				results[uemail] = 1
				rawemails = list(set(rawemails))

	for key in results.keys():
		print key
		buffer.write(key + '\n')

	print '\n  Found %s email(s).' % str(len(results))
	buffer.write('\n  Found ' + str(len(results)) + ' email(s).\n')

def save_output(ofile):
	try:
		log = open(ofile, 'w')
		log.write('# Logfile: ' + time.strftime('%a, %d %b %Y, %H:%M:%S\n\n'))
		log.write(buffer.getvalue())
	except IOError:
		print 'No such file or directory: \'%s\'.' % ofile
		sys.exit(1)
	finally:
		log.close()

def main(argv):
	global domain
	ifile = 'subdomain.plugin'
	ofile = ''

	if len(sys.argv) < 2:
		usage()
		sys.exit(1)

	try:
		options, arguments = getopt.getopt(argv, 'm:d:i:o:', ['mode=', 'domain=', 'input=', 'output='])
	except getopt.GetoptError:
		print 'Option not recognized. Try again.'
	#	usage()
		sys.exit(1)

	for option, argument in options:
		if option in ('-m', '--mode'):
			mode = str(argument)
		elif option in ('-d', '--domain'):
			domain = str(argument)
		elif option in ('-i', '--input'):
			ifile = str(argument)
		elif option in ('-o', '--output'):
			ofile = str(argument)

	if mode == 's' and ofile == '':
		find_subdomains(ifile)
	elif mode == 's' and ofile != '':
		find_subdomains(ifile); save_output(ofile)

	if mode == 'z' and ofile == '':
		zone_transfer()
	elif mode == 'z' and ofile != '':
		zone_transfer(); save_output(ofile)

	if mode == 'e' and ofile == '':
		generate_urls(); find_emails(urls)
	elif mode == 'e' and ofile != '':
		generate_urls(); find_emails(urls); save_output(ofile)

if __name__ == '__main__':
	try:
		main(sys.argv[1:])
	except KeyboardInterrupt:
		print '\nInterrupted by user...'
		sys.exit(1)
